(in-package "USER")

(defvar *relations*)

(defvar *atom-threshold*)

(defvar *eq-threshold*)

(defvar *and-threshold*)

(defvar *or-threshold*)

(defvar *all-threshold*)

(defvar *ex-threshold*)

(defvar *eq-guard-threshold*)

(defvar *current-var*)

(defvar *relation-number*)

(defun initialize-generator ()
  
  (setf *relations* (make-array 12))
  
  (setf (aref *relations* 0) ':top
	(aref *relations* 1) 'p
	(aref *relations* 2) 'r
	(aref *relations* 3) 's
	(aref *relations* 4) 't
	(aref *relations* 5) 'u
	(aref *relations* 6) 'v
	(aref *relations* 7) 'w
	(aref *relations* 8) 'x
	(aref *relations* 9) 'y
	(aref *relations* 10) 'z
	
	*atom-threshold* 50
	*eq-threshold* 100
	
	*and-threshold* 25
	*or-threshold* 50
	*all-threshold* 75
	*ex-threshold* 100
	
	*eq-guard-threshold* 20
	
	*current-var* 2
	
	*relation-number* 3
	))

(defun generate-bench-formula (width depth)
  "generates a formula with width and dept and the necessary successors"
  
  (let (formula conjunct arity relation)
  
    (dotimes (counter width)
      (setf conjunct (generate-formula depth '((? 1))))
      (push conjunct formula))
    
    (dotimes (counter *relation-number*)
      
      (setf arity (+ 1 counter))
      (setf relation (aref *relations* arity))
      
      (unless (eq 1 arity)
	(dotimes (position arity)
	  (let (guard new-var new-vars ex-form 
		all-var all-vars all-guard all-form)
	    (setf all-var (get-new-var))
	      
	    (dotimes (number arity)
	      
	      (if (eq number position)
		  (push all-var guard)
		(progn
		  (setf new-var (get-new-var))
		  (push new-var new-vars)
		  (push new-var guard))))
	    
	    (push relation guard)
	    
	    (setf ex-form (list :ex new-vars guard '(:top)))
	    
	    (setf all-vars (list all-var)
		  all-guard (list :eq all-var all-var)
		  all-form (list :all all-vars all-guard ex-form))
	    
	    (push all-form formula)))))
    
    (push :and formula)
    
    formula))
	    

(defun generate-formula (depth vars)
  "creates a formula of a certain depth 
IN depth, current variables
OUT gf-formula"
  
  (let ((kind (random 100)))
      
    (if (eq depth 0)
	(cond
	 ((< kind *atom-threshold*) (generate-atom vars))
	 ((< kind *eq-threshold*) (generate-eq vars)))
      
      (cond
       ((< kind *and-threshold*) (generate-and depth vars))
       ((< kind *or-threshold*) (generate-or depth vars))
       ((< kind *all-threshold*) (generate-all depth vars))
       ((< kind *ex-threshold*) (generate-ex depth vars))))))

(defun generate-variables (number vars)
  
  (let ((clength (length vars))
	clist var cnumber)
    
    (dotimes (index number clist)
      (setf cnumber (random clength)
	    var (nth cnumber vars)
	    clist (cons var clist)))))
      

(defun generate-atom (vars)
  
  (let* ((negated (random 2))
	 (relation (random *relation-number*))
	 (variable-list (generate-variables relation vars))
	 (formula (cons (aref *relations* relation) variable-list)))
    
    (if (< negated 1)
	(list :not formula)
      formula)))

(defun generate-eq (vars)
  
  (let* ((negated (random 2))
	 (variable-list (generate-variables 2 vars))
	 (formula (cons :eq variable-list)))
    
    (if (< negated 1)
	(list :not formula)
      formula)))

(defun generate-and (depth vars)
  
  (list :and 
	(generate-formula (- depth 1) vars)
	(generate-formula (- depth 1) vars)))

(defun generate-or (depth vars)
  
  (list :or 
	(generate-formula (- depth 1) vars)
	(generate-formula (- depth 1) vars)))

(defun generate-all (depth vars)
  
  (let ((arity (+ 1 (random *relation-number*)))
	(equality (random 100))
	new-var var-list new-vars guard)
    
    (if (< equality *eq-guard-threshold*)
	(progn
	  (setf new-var (get-new-var))
	  (setf new-vars (list new-var))
	  (setf var-list (list new-var))
	  (setf guard (list :eq new-var new-var)))

      (progn
	(dotimes (var arity)
	  (if (or
	       (and (null new-vars) (eq var (- arity 1))) ; at least one new var
	       (eq 0 (length vars))	; no old vars
	       (< 1 (random 2)))	; probability old-new 50:50
	      (progn
		(setf new-var (get-new-var))
		(push new-var var-list)
		(push new-var new-vars))
	    (push (get-old-var vars) var-list)))
	(setf guard (cons (get-relation arity) var-list))))
    
    (list :all 
	  new-vars			; ((? 10) (? 11))
	  guard				; (S (? 1) (? 10) (? 11))
	  (generate-formula (- depth 1) var-list)))) ; f((? 1) (? 10) (? 11))
  
(defun generate-ex (depth vars)
  
  (let ((arity (+ 1 (random *relation-number*)))
	new-var var-list new-vars guard)
    
    (dotimes (var arity)
      (if (or
	   (and (null new-vars) (eq var (- arity 1))) ; at least one new var
	   (eq 0 (random 2)))		; probability old-new 50:50
	  (progn
	    (setf new-var (get-new-var))
	    (push new-var var-list)
	    (push new-var new-vars))
	(push (get-old-var vars) var-list)))
    
    (setf guard (cons (get-relation arity) var-list))    
    
    (list :ex 
	  new-vars			; ((? 10) (? 11))
	  guard				; (S (? 1) (? 10) (? 11))
	  (generate-formula (- depth 1) var-list)))) ; f((? 1) (? 10) (? 11))
  

(defun get-new-var ()
  
  (list '? (incf *current-var*)))

(defun get-old-var (var-list)
  
  (nth (random (length var-list)) var-list))

(defun get-relation (arity)
  
  (aref *relations* arity))

(defun generate-bench-file (width depth relations number)
  (let (formula filename)
    
    (setf filename (with-output-to-string (stream)
      (format stream "gf-w~d-d~2,'0d-r~d.~d" width depth relations number)))
    
    (format t "~s~%" filename)
  
    (setf *relation-number* relations)
  
    (with-open-file (outstream filename 
		     :direction :output
		     :if-exists :overwrite
		     :if-does-not-exist :create)
      (with-standard-io-syntax
	(setf *current-var* 2)
	(setf formula (generate-bench-formula width depth ))
	(prin1 formula outstream)))))


(defun bench-set ()
  
  (dolist (width '(4 8))
    (dolist (depth '(8 16))
      (dolist (relations '(2 4))
	(dotimes (counter 8)
	  
	  (generate-bench-file width depth relations counter))))))



    
