(in-package "USER")

(defun recomp ()
  
  (compile-file "saga.lisp")
  (compile-file "saga-bench-gf.lisp")
  
  (load "saga.fasl")
  (load "saga-bench-gf.fasl"))
 
(defvar *begin*)
(defvar *maxtime*)
(defvar *listfile*)
(defvar *protocol-file*)
(defvar *verbose*)
(defvar *verbose-file*)

(setf *maxtime* 100)
(setf *listfile* "gf-hbench-list")
(setf *protocol-file* "gf-hbench-results-2.0.txt")
(setf *verbose-file* "gf-hbench-details-2.0.txt")
(setf *verbose* t)

(setf *simplify* t
      *moms* nil
      *imoms* t
      *backjump* t
      *sem-branch* t
      *blocking* t
      *subset-blocking* nil
      *sequence* '(atom and eq all ex or))
      

(setf *early-flatten* t)

(defun fullgfbench ()
  
  (dolist (sim '(t nil))
    (dolist (sequence '((atom and eq all ex or) (atom and eq all or ex)))
      (dolist (sb '(t nil))
	(dolist (moms '(t nil))
	  (dolist (imoms '(t nil))
	    (dolist (bj '(t nil))
	      (setf *sequence* sequence
		    *simplify* sim
		    *moms* moms
		    *imoms* imoms
		    *backjump* bj
		    *sem-branch* sb)
	      
	      (unless (or
		       ;; temp!
		       (and (eq sim t)
			    (equal sequence '(atom and eq all ex or)))
		       
		       (and (not *sem-branch*) *moms*) ; moms without sb
		       (and (not *moms*) *imoms*)) ; imoms witout moms
		
		(let ((number (gfbench)))
		  
		  (when *verbose*
		    (with-open-file
			(outstream *verbose-file* 
			 :direction :output
			 :if-exists :append
			 :if-does-not-exist :create)
		      (format outstream "~a " number)
		    
		      (format outstream 
			      " seq ~a simpl ~a sb ~a moms ~a imoms ~a bj ~a~%" 
			      sequence sim sb moms imoms bj)))
		  
		  (with-open-file
		      (outstream *protocol-file* 
		       :direction :output
		       :if-exists :append
		       :if-does-not-exist :create)
		    (format outstream "~a " number)
		    
		    (format outstream 
			    " seq ~a simpl ~a sb ~a moms ~a imoms ~a bj ~a~%" 
			    sequence sim sb moms imoms bj)))))))))))

(defun gfbench ()
  (excl:advise satisfy :before watchdog nil 
	       (when (> (- (get-internal-run-time) *begin*) 
			(* *maxtime* internal-time-units-per-second))
					;		 (print "out of time") 
		 (throw 'timeout nil)))
  
  (let  ((total-number 0))
  (with-open-file (in-stream *listfile* :direction :input)
    (do ((nextline (read-line in-stream nil nil)
		   (read-line in-stream nil nil)))
	((null nextline) nil)
      (let* ((formula-file nextline)
	     (saga-formula 
	      (with-open-file (f-stream formula-file :direction :input) 
		(read f-stream nil nil)))
	     satisfiable begin end time timeout out-of-mem result)
	
	(excl:gc t)			; global gc
	(setf begin (get-internal-run-time))
	(setf *begin* begin)
	
	(handler-case 
	    (catch 'timeout
	      (setf satisfiable (construct-tableau saga-formula)))
	  (storage-condition () (setf out-of-mem t)))
	
	(setf end (get-internal-run-time))
	(setf time (round (/ (- end begin) internal-time-units-per-second)))
	
	(if (>= time *maxtime*)
	    (setf timeout t)
	  (setf timeout nil))
	(setf result 
	  (cond (timeout "T")
		(out-of-mem "M")
		(satisfiable "S")
		(t "U")))
	
	(when *verbose*
	  (with-open-file
	      (outstream *verbose-file* 
	       :direction :output
	       :if-exists :append
	       :if-does-not-exist :create)
	    (format outstream "i ~a ~a ~a~%" nextline time result)))
	
	(unless (or timeout out-of-mem)
	  (incf total-number)))))
  total-number))
